# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Celery tasks for `invenio-moodle`."""


from celery import shared_task
from flask import current_app
from invenio_access.permissions import system_identity

from .proxies import current_moodle


@shared_task(ignore_result=True)
def try_fetch_moodle_except_mail() -> None:
    """Fetch data from moodle and enter it into database."""
    import_func = current_app.config["MOODLE_IMPORT_FUNC"]
    moodle_service = current_moodle.moodle_rest_service

    records = moodle_service.fetch_records(system_identity)

    for record in records:
        try:
            import_func(system_identity, record, moodle_service)
        except RuntimeError as error:
            msg = "ERROR moodle import error: %s"
            current_app.logger.error(msg, str(error))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Default configuration for invenio-moodle."""

import sys

import click

MOODLE_CELERY_BEAT_SCHEDULE = {}
"""The celery beat schedule.

This variable is used to configure the celery task which is used to import data
from moodle on a regularly basis.
"""


MOODLE_ENDPOINT = ""
"""The url of the moodle endpoint from where should be fetched the metadata."""


def default_import_func(*_: dict, **__: dict) -> None:
    """Define the default import func."""
    click.secho("Please set the variable MOODLE_REPOSITORY_IMPORT_FUNC", fg="yellow")
    sys.exit()


MOODLE_REPOSITORY_IMPORT_FUNC = default_import_func



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Provides API for Moodle."""

from .ext import InvenioMoodle
from .services import MoodleRESTService

__version__ = "1.0.0"

__all__ = ("__version__", "InvenioMoodle", "MoodleRESTService")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Provides API to Moodle."""

from setuptools import setup

setup()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 Graz University of Technology.
#
# invenio-moodle is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Records."""

from dataclasses import dataclass
from pathlib import Path
from shutil import copyfileobj
from tempfile import NamedTemporaryFile, _TemporaryFileWrapper

from requests import HTTPError, get, head

from .types import URL


@dataclass
class MoodleRESTConfig:
    """Moodle rest config."""

    endpoint: str = ""


class MoodleConnection:
    """Moodle connection."""

    def __init__(self, config: MoodleRESTConfig) -> None:
        """Construct."""
        self.config = config

    def get(self) -> dict:
        """Get."""
        try:
            response = get(self.config.endpoint, timeout=10)
            response.raise_for_status()
        except HTTPError as error:
            raise RuntimeError(str(error)) from error
        else:
            return response.json()

    def get_filename(self, file_url: URL) -> str:
        """Get filename."""
        headers = head(file_url, timeout=10).headers
        try:
            return headers["Content-Disposition"].split("filename=")[-1].strip('"')
        except KeyError as error:
            msg = f"ERROR moodle no filename found for url: {file_url}"
            raise RuntimeError(msg) from error

    def store_file_temporarily(
        self,
        file_url: URL,
        file_pointer: _TemporaryFileWrapper,
    ) -> None:
        """Store file temporarily."""
        with get(file_url, stream=True, timeout=10) as response:
            copyfileobj(response.raw, file_pointer)


class MoodleAPI:
    """Moodle api."""

    connection_cls = MoodleConnection

    def __init__(self, config: MoodleRESTConfig) -> None:
        """Construct."""
        self.connection = self.connection_cls(config)

    def download_file(self, file_url: URL) -> str:
        """Download file."""
        filename = self.connection.get_filename(file_url)
        prefix = Path(filename).stem
        suffix = Path(filename).suffix

        with NamedTemporaryFile(
            delete=False,
            delete_on_close=False,
            prefix=f"{prefix}-",
            suffix=suffix,
        ) as file_pointer:
            self.connection.store_file_temporarily(file_url, file_pointer)
        return file_pointer.name

    def fetch_records(self) -> dict:
        """Fetch data from the endpoint."""
        return self.connection.get()
